@@ -55,6 +55,16 @@
 
 int compat_mode = 1; /* Start compatible with fdisk and cfdisk */
 
+/* This variable is global and referred from fdisk.c and cfdisk.c
+ * The value specifies how the partitions will be aligned,
+ * values of ALIGNMENT_OPTIMAL and ALIGNMENT_MINIMAL are handled
+ * by the functions in this file, while ALIGNMENT_CYLINDER is 
+ * handled by fdisk.c and cfdisk.c.
+ */
+
+AlignType current_partition_align_type = ALIGNMENT_OPTIMAL;
+
+
 /* Here we store the struct with interface functions */
 static UICalls *uiquery;
 
@@ -162,8 +172,97 @@ _can_create_logical (const PedDisk* disk)
 	return ped_disk_extended_partition (disk) != 0;
 }
 
+/* This function returns a device constraint that is optionally
+ * aligned if partition_align_type is either ALIGNMENT_OPTIMAL
+ * or ALIGNMENT_MINIMAL.
+ */
+static PedConstraint *
+_get_aligned_device_constraint(PedDevice *dev)
+{
+    if (partition_align_type == ALIGNMENT_OPTIMAL)
+        return ped_device_get_optimal_aligned_constraint(dev);
+    else if (partition_align_type == ALIGNMENT_MINIMAL)
+        return ped_device_get_minimal_aligned_constraint(dev);
+    else
+        return ped_device_get_constraint(dev);
+}
 
 
+/* Return a new geometry aligned according to partition_align_type.
+ * Returns NULL if it didn't succeed.
+ */
+static PedGeometry *
+_get_aligned_geometry(PedGeometry *geometry, int keep_inside)
+{
+	PedConstraint *constraint;
+	PedGeometry * new;
+
+	constraint = _get_aligned_device_constraint(geometry->dev);
+	if (keep_inside) {
+		constraint = constraint_intersect_and_destroy(
+				constraint,
+				ped_constraint_new_from_max(geometry));
+	}
+
+	new = ped_constraint_solve_nearest(constraint, geometry);
+	ped_constraint_destroy(constraint);
+	return new;
+}
+
+/* Returns 1 if the partition is aligned correctly in regards to
+ * align_type.
+ */
+static int
+_is_partition_aligned(PedPartition *part, AlignType align_type)
+{
+    
+    PedAlignment *alignment = NULL;
+
+    switch (partition_align_type) {
+        case ALIGNMENT_OPTIMAL:
+            alignment = ped_device_get_optimum_alignment (part->disk->dev);
+            break;
+        case ALIGNMENT_MINIMAL:
+            alignment = ped_device_get_minimum_alignment (part->disk->dev);
+            break;
+				default:
+						;
+    }
+        
+    if (!alignment)
+        return 1;
+
+    int aligned = 1;
+    if (alignment->grain_size == 0)
+        aligned = 0;
+    else if (part->geom.start % alignment->grain_size != alignment->offset)
+        aligned = 0;
+    ped_alignment_destroy(alignment);
+
+    return aligned;
+}
+
+/* Warn the user if the partition isn't aligned correctly 
+ * as specified by the partition_align_type global.
+ * Returns 1 if the partition is correctly aligned, or
+ * if the user accepted the partition as it is. Otherwise,
+ * return 0.
+ */
+static int
+_partition_warn_unaligned(PedPartition *part)
+{
+    if (_is_partition_aligned(part, partition_align_type))
+        return 1;
+
+    if (ped_exception_throw(PED_EXCEPTION_WARNING, 
+                            PED_EXCEPTION_IGNORE_CANCEL,
+                            _("The resulting partition is not properly "
+                              "aligned for best performance"))
+            == PED_EXCEPTION_CANCEL)
+        return 0;
+
+    return 1;                            
+}
 
 static int
 _partition_warn_busy (PedPartition* part)
@@ -439,7 +538,7 @@ fuzzify (PedGeometry *geom, PedDevice *dev, PedConstraint *constraint,
 		constraint = ped_device_get_constraint(dev);
 	PedSector start = geom->start-before;
 	start = (start > 0 ? start : 0);
-	PedSector end = geom->start+after;
+	PedSector end = geom->end+after;
 	end = (end < dev->length-1LL ? end : dev->length-1LL);
 	ped_geometry_set (geom, start, end-start+1);
 	new_geom = ped_constraint_solve_nearest (constraint, geom);
@@ -507,6 +606,29 @@ fix_part_position(PedDevice *dev, PedSector *start, PedSector *end,
 
 }
 
+AlignType 
+string_to_align_type(const char *name)
+{
+    if (!strcmp(name, "none"))
+        return ALIGNMENT_NONE;
+    if (!strncmp(name, "opt", 3))
+        return ALIGNMENT_OPTIMAL;
+    if (!strncmp(name, "min", 3))
+        return ALIGNMENT_MINIMAL;
+    if (!strncmp(name, "cyl", 3))
+        return ALIGNMENT_CYLINDER;
+    return ALIGNMENT_INVALID;
+}
+
+int
+init_disk_flags(PedDisk *disk)
+{
+    if (ped_disk_is_flag_available(disk, PED_DISK_CYLINDER_ALIGNMENT))
+        if (!ped_disk_set_flag(disk, PED_DISK_CYLINDER_ALIGNMENT,
+                               partition_align_type == ALIGNMENT_CYLINDER))
+            return 0;
+    return 1;
+}
 
 /* PedSector is signed, so we can use this to move in both directions */
 static int
@@ -658,31 +780,43 @@ query_part_position(const char* prompt, const void* context, PartPos *pos,
 			   the sector with the range so that the sector is equal
 			   to the desired result */
 			if (where == 'b') {
-				/* As the start is not equal to the end of
-				   the partition, we will add some fuzz */
-				/* FIXME */
-				pos->start.range = ped_geometry_new(dev,
-							first, 64LL);
-
+				/* Fuzzing both start and end with range->length. 
+				 * TODO: range->length / 2 might be a better choice
+				 *       ...or a constraint with a range for min and max
+				 *       size based on this constraint which isn't possible
+				 *       with the API right now.
+				 */
+				pos->start.range = ped_geometry_new(dev, first, 1);
+				fuzzify(pos->start.range, dev, NULL, 
+					range->length, range->length);
 				pos->start.sector = first;
+
 				/* Desired: end = first+length-1LL */
 				pos->end.sector = length;
 				pos->end.range = range;
 				move_sector(dev,&(pos->end),first-1LL);
 			}
 			else if (where == 'e') {
-				/* If the end has changed, we will add some fuzz */
-				if (pos->end.sector != last)
-					pos->end.range = ped_geometry_new(dev,
-							last-63LL, 64LL);
-
+				/* Fuzzing both start and end with range->length */
+				pos->end.range = ped_geometry_new(dev, last, 1);
+				fuzzify(pos->end.range, dev, NULL, 
+					range->length, range->length);
 				pos->end.sector = last;
+
 				/* Desired: start = last-length+1LL */
 				pos->start.sector = length;
 				pos->start.range = range;
 				move_sector(dev,&(pos->start),last-2*length+1LL);
 			}
 			else if (where == 's') {
+				/* If UI_SNAP_TO_START isn't true, add fuzz to the
+				 * start, otherwise don't.
+				 */
+				pos->start.range = ped_geometry_new(dev, pos->start.sector, 1);
+				if (!(*opts & UI_SNAP_TO_START))
+					fuzzify(pos->start.range, dev, NULL, 
+						range->length, range->length);
+
 				/* Desired: end = start+length-1LL */
 				pos->end.sector = length;
 				pos->end.range = range;
@@ -729,6 +863,10 @@ get_disk (const char* prompt, PedDisk** value)
 		PedDisk* new_disk = ped_disk_new (dev);
 		if (!new_disk)
 			return 0;
+		if (!init_disk_flags(new_disk)) {
+			ped_disk_destroy(new_disk);
+			return 0;
+		}
 		*value = new_disk;
 	}
 	return 1;
@@ -1071,15 +1209,19 @@ get_sector (const char* prompt, PedDevice* dev, SectPos *value,
 }
 
 /* Looks for the first suitable free space for a new partition */
-place_part_start(PedDisk *disk, PartPos *pos, PedPartitionType type) {
+int place_part_start(PedDisk *disk, PartPos *pos, PedPartitionType type) {
+	PedGeometry *aligned;
 	PedPartition *part = NULL;
 	type = (type & PED_PARTITION_LOGICAL) | PED_PARTITION_FREESPACE;
 	for (part = ped_disk_next_partition(disk, NULL);
 	     part; part = ped_disk_next_partition(disk,part)) {
 		if (part->type == type) {
-			pos->start.sector = part->geom.start;
+			aligned = _get_aligned_geometry(&part->geom, 1);
+
+			pos->start.sector = aligned->start;
 			pos->start.range = ped_geometry_new (disk->dev,
-				 		part->geom.start, 8LL);
+				 		aligned->start, 1);
+			ped_geometry_destroy(aligned);
 			return 1;
 		}
 	}
@@ -1089,12 +1231,16 @@ place_part_start(PedDisk *disk, PartPos *pos, PedPartitionType type) {
 /* This function looks where to place the end of a new partition */
 static int
 place_part_end(PedDisk *disk, PartPos *pos) {
+	PedGeometry *aligned;
 	PedPartition *part = ped_disk_get_partition_by_sector(disk,
 							pos->start.sector);
 	if (part->type & PED_PARTITION_FREESPACE) {
-		pos->end.sector = part->geom.end;
+		aligned = _get_aligned_geometry(&part->geom, 1);
+
+		pos->end.sector = aligned->end;
 		pos->end.range = ped_geometry_new (disk->dev,
-				 	part->geom.end - 7LL, 8LL);
+				 	aligned->end, 1);
+		ped_geometry_destroy(aligned);
 		return 1;
 	}
 	else
@@ -1132,7 +1278,7 @@ get_position(PartPos* pos, PedDisk *disk, GSOpts opts)
   
   if (!get_sector (start, disk->dev, &(pos->start), -1LL, opts))
     return 0;
-  
+
   /* If we are creating a new partition and want to look where to
      place its end ... */
   if (opts & PLACE_END_NEWPART)
@@ -1291,10 +1437,17 @@ perform_mklabel (PedDevice* dev, PedDisk** disk, const PedDiskType* type)
 	      goto error;
 	  }
 
-        *disk = ped_disk_new_fresh (dev, type);
-        if (!*disk)
+        PedDisk *new_disk = ped_disk_new_fresh (dev, type);
+        if (!new_disk)
 	  goto error;
 
+	if (!init_disk_flags(new_disk)) {
+		ped_disk_destroy(new_disk);
+		goto error;
+	}
+
+	*disk = new_disk;
+
         /*if (!ped_disk_commit (*disk))
                 goto error;*/
 	uiquery->need_commit = 1;
@@ -1448,8 +1601,6 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
 	/* Check if the partition occupying the space before the place we
 	   are trying to put this one, requires a metadata sectors after it,
 	   and if so, increase the range of the constraint */
-
-
 	fuzzify(pos->start.range, disk->dev, NULL, 0,
 		metadata_tail_sectors(
 			disk_get_prev_nmd_partition(
@@ -1470,7 +1621,7 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
                         pos->start.range, pos->end.range);
         PED_ASSERT (user_constraint != NULL, return 0);
 
-        dev_constraint = ped_device_get_constraint (disk->dev);
+        dev_constraint = _get_aligned_device_constraint(disk->dev);
         PED_ASSERT (dev_constraint != NULL, return 0);
 
         final_constraint = ped_constraint_intersect (user_constraint,
@@ -1490,10 +1641,10 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
                         end_usr   = ped_unit_format (disk->dev, pos->end.sector);
                         start_sol = ped_unit_format (disk->dev, part->geom.start);
                         end_sol   = ped_unit_format (disk->dev, part->geom.end);
-			/* TODO: With the changes from 06/08/06, we might remove
-			   the "if" and warn the user always if this is out of
-			   range */
-			if (opts & UI_CUSTOM_VALUES)
+			if (!ped_geometry_test_sector_inside(pos->start.range, 
+			                                     part->geom.start) ||
+			    !ped_geometry_test_sector_inside(pos->end.range,
+			    				     part->geom.end))
 				switch (ped_exception_throw (
 					PED_EXCEPTION_WARNING,
 					PED_EXCEPTION_YES_NO,
@@ -1512,6 +1663,8 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
 						/* undo partition addition */
 						goto error_remove_part;
 				}
+			if (!_partition_warn_unaligned(part))
+				goto error_remove_part;
                 } else {
                         goto error_remove_part;
                 }
@@ -1594,12 +1747,12 @@ error_remove_part:
 	/* We need to make this NULL here, for cfdisk's usage */
 	if (newpart) *newpart = NULL;
         ped_disk_remove_partition (disk, part);
-error_destroy_all_constraints:
+//error_destroy_all_constraints:
         ped_constraint_destroy (final_constraint);
 error_destroy_simple_constraints:
         ped_constraint_destroy (user_constraint);
         ped_constraint_destroy (dev_constraint);
-error_destroy_part:
+//error_destroy_part:
         ped_partition_destroy (part);
 error:
         if (pos->start.range != NULL)
@@ -1865,6 +2018,8 @@ _rescue_add_partition (PedPartition* part)
         switch (ex_opt) {
                 case PED_EXCEPTION_CANCEL: return -1;
                 case PED_EXCEPTION_NO: return 0;
+								default: 
+																			 ;
         }
 
         ped_partition_set_system (part, fs_type);
@@ -1937,11 +2092,11 @@ _rescue_pass (PedDisk* disk, PedGeometry* start_range, PedGeometry* end_range)
 
 error_remove_partition:
         ped_disk_remove_partition (disk, part);
-error_partition_destroy:
+//error_partition_destroy:
         ped_partition_destroy (part);
-error_constraint_done:
+//error_constraint_done:
         ped_constraint_done (&constraint);
-error:
+//error:
         return 0;
 }
 
@@ -2514,7 +2669,6 @@ _print_flag (int *i, const char *name) {
 void
 print_partition_types() {
 	PedFileSystemType *walk;
-	char buf[SMALLBUF];
 	for (walk = ped_file_system_type_get_next(NULL); walk; walk = ped_file_system_type_get_next(walk)) {
 		int i = 0;
 		printf ("%-15s",walk->name);
