@@ -48,6 +48,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <limits.h>
+#include <errno.h>
 
 #ifdef ENABLE_MTRACE
 #include <mcheck.h>
@@ -110,9 +111,12 @@ struct {
 #endif /* GNU_EXT */
   { "interactive", no_argument, NULL, 'i', "where necessary, prompts for user intervention"},
   { "script", no_argument, NULL, 'p', "never prompts for user intervention"},
+#ifdef GNU_EXT
+  { "align", required_argument, NULL, 'A', "specify align to be none, cyl, min or opt"},
+#endif
   { "sector-units", no_argument, NULL, 'u', "use sectors instead of cylinder as a default unit"},
   { "sector-size", required_argument, NULL, 'b', "specify the sector size in bytes"},
-  { "cilyndres", required_argument, NULL, 'C', "specify the number of cylinders, actually does nothing"},
+  { "cylinders", required_argument, NULL, 'C', "specify the number of cylinders, actually does nothing"},
   { "heads", required_argument, NULL, 'H', "in lfdisk, specify the number of heads of the disk"},
   { "sectors", required_argument, NULL, 'S', "in lfdisk, specify the number of sectors per track"},
   { "list-partition-types", no_argument, NULL, 't', "displays a list of supported partition types"},
@@ -129,7 +133,7 @@ struct {
  * to this string 
  *
  * NOTE: I will make function to build dynamic in the future */
-static const char* short_option_string = "hlipvLGs:utb:C:H:S:r";
+static const char* short_option_string = "hlipvLGs:A:utb:C:H:S:r";
 
 
 
@@ -156,15 +160,17 @@ static char* number_msg = N_(
 static char* label_type_msg_start = N_("LABEL-TYPE is one of: ");
 static char* flag_msg_start =	N_("FLAG is one of: ");
 static char* unit_msg_start =	N_("UNIT is one of: ");
-static char* part_type_msg =	N_("PART-TYPE is one of: primary, logical, "
-			           "extended\n");
+// unused
+//static char* part_type_msg =	N_("PART-TYPE is one of: primary, logical, "
+//			           "extended\n");
 static char* fs_type_msg_start = N_("FS-TYPE is one of: ");
 static char* start_end_msg =	N_("START and END are disk locations, such as "
 		"4GB or 10%.  Negative values count from the end of the disk.  "
                 "For example, -1s specifies exactly the last sector.\n");
-static char* state_msg =	N_("STATE is one of: on, off\n");
-static char* device_msg =	N_("DEVICE is usually /dev/hda or /dev/sda\n");
-static char* name_msg =		N_("NAME is any word you want\n");
+// unused
+//static char* state_msg =	N_("STATE is one of: on, off\n");
+// static char* device_msg =	N_("DEVICE is usually /dev/hda or /dev/sda\n");
+// static char* name_msg =		N_("NAME is any word you want\n");
 static char* resize_msg_start = N_("The partition must have one of the "
 				   "following FS-TYPEs: ");
 
@@ -520,6 +526,8 @@ constraint_intersect_and_destroy (PedConstraint* a, PedConstraint* b)
 	return result;
 }
 #endif
+// unused 
+#if 0
 static void
 help_on (char* topic)
 {
@@ -530,7 +538,7 @@ help_on (char* topic)
 
         fdisk_command_print_help (cmd);
 }
-
+#endif
 
 static int
 _disk_reread(PedDisk **disk) {
@@ -539,9 +547,13 @@ _disk_reread(PedDisk **disk) {
 
 	ped_disk_destroy(*disk);
 
-	*disk = ped_disk_new(dev);
-	uiquery.need_commit = 0;
+	if((*disk = ped_disk_new(dev)) == NULL)
+		return 0;
 
+	init_disk_flags(*disk);
+	uiquery.need_commit = 0;
+	
+	return 1;
 }
 
 static int
@@ -550,11 +562,13 @@ do_check (PedDisk** disk)
 	return perform_check(*disk,NULL);
 }
 
+#if 0
 static int
 do_cp (PedDisk** disk)
 {
 	return perform_cp(*disk,NULL,UI_WARN_COMMIT);
 }
+#endif
 
 void
 fdisk_print_options_help ()
@@ -634,6 +648,8 @@ do_dvh_mklabel (PedDisk** disk)
 	return perform_mklabel(dev,disk,type);
 }
 
+// unused
+#if 0
 static int
 do_mklabel (PedDisk** disk)
 {
@@ -645,7 +661,7 @@ do_mklabel (PedDisk** disk)
 
 	return perform_mklabel(dev,disk,type);
 }
-
+#endif
 static int
 do_mkfs (PedDisk** disk)
 {
@@ -657,7 +673,6 @@ static int
 do_mkpart (PedDisk **disk)
 {
 	PartPos			 pos;
-	PedPartition* 		 part;
 	PedPartitionType	 part_type = 0;
 	const PedFileSystemType	*fs_type = ped_file_system_type_get ("ext2");
 	char*			 peek_word;
@@ -757,12 +772,14 @@ do_move (PedDisk** disk)
 	}
 	return 1;
 }
-
+// unused
+#if 0
 static int
 do_name (PedDisk** disk)
 {
 	return perform_name(*disk, NULL, NULL);
 }
+#endif
 
 /* TODO: REMOVE */
 #if 0
@@ -786,6 +803,7 @@ partition_print_flags (PedPartition* part)
 }
 #endif
 
+#if 0
 /* Prints a sector out, first in compact form, and then with a percentage.
  * Eg: 32Gb (40%)
  */
@@ -808,7 +826,9 @@ print_sector_compact_and_percent (PedSector sector, PedDevice* dev)
 	PED_FREE (compact);
 	PED_FREE (percent);
 }
+#endif
 
+#if 0
 static int
 partition_print (PedPartition* part)
 {
@@ -843,6 +863,7 @@ partition_print (PedPartition* part)
 	ped_file_system_close (fs);
 	return 1;
 }
+#endif
 
 static int
 do_quit (PedDisk** disk)
@@ -939,6 +960,7 @@ do_edit_bsd_disklabel (PedDisk** disk)
 			label_disk = ped_disk_new_fresh (label_dev,
 					ped_disk_type_get ("bsd"));
 		}
+		init_disk_flags(label_disk);
 		in_menu = 2;
 		logical_offset = count_logical_partition(*disk);
 		fdisk_interactive_menu(&label_disk, fdisk_bsd_menu_commands, 2);
@@ -1125,9 +1147,6 @@ do_print (PedDisk **disk)
 	  do_quit (disk);*/
 
 	return 1;
-
-error:
-	return 0;
 }
 
 #define sector(s)	((s) & 0x3f)
@@ -1258,7 +1277,7 @@ do_raw (PedDisk **disk)
 	{
 		print_sector((*disk)->dev, part->prev->geom.start);
 	}
-
+	return 1;
 }
 
 void
@@ -1272,7 +1291,6 @@ fdisk_print_partition_size(PedDisk *disk)
 	else {
 		PedCHSGeometry* 	chs;
 		PedSector               sectors;
-		PedSector		cyl_size;
 		PedSector		cyl_start;
 		PedSector		cyl_end;
 		PedSector		sect_size;
@@ -1313,6 +1331,7 @@ fdisk_do_list_devices (PedDisk* disk) {
 			if (!disk) // Not fatal error
 				continue;
 
+			init_disk_flags(disk);
 			if (fdisk_print_raw)
 				do_raw (&disk);
 			else
@@ -1329,6 +1348,7 @@ fdisk_do_list_devices (PedDisk* disk) {
         exit(0);
 }
 
+#if 0
 static PedPartitionType
 _disk_get_part_type_for_sector (PedDisk* disk, PedSector sector)
 {
@@ -1341,6 +1361,8 @@ _disk_get_part_type_for_sector (PedDisk* disk, PedSector sector)
 
 	return PED_PARTITION_LOGICAL;
 }
+#endif
+
 #if 0
 /* This function checks if "part" contains a file system, and returs
  * 	0 if either no file system was found, or the user declined to add it.
@@ -1532,7 +1554,7 @@ do_resize (PedDisk** disk)
 	PedSector first,last;
 	PedConstraint *constraint = NULL;
 	PedPartitionType desired;
-	UIOpts ui_opts = UI_WARN_COMMIT;
+	UIOpts ui_opts = UI_WARN_COMMIT | UI_SNAP_TO_START;
 
 	if (!get_partition (_("Partition"), *disk, &part))
                 	return 0;
@@ -1685,6 +1707,7 @@ do_rm (PedDisk** disk)
 	return perform_rm (*disk, NULL);
 }
 
+#if 0
 static int
 do_select (PedDisk** disk)
 {
@@ -1705,6 +1728,14 @@ do_select (PedDisk** disk)
 	if (!(*disk = ped_disk_new (dev)))
 	        return 0;
 
+	/* Initialize the disk flags */
+	if (!init_disk_flags(*disk)) {
+		ped_disk_destroy(*disk);
+		*disk = NULL;
+		return 0;
+	}
+
+
 	/* Tell the user we are using the new device. */
 	fdisk_print_using_dev (dev);
 
@@ -1712,13 +1743,15 @@ do_select (PedDisk** disk)
 
 	return 1;
 }
+#endif
 
+#if 0
 static int
 do_set (PedDisk** disk)
 {
   return perform_set (*disk, NULL, 0, UI_DEFAULT);
 }
-
+#endif
 static int
 do_toggle_boot_flag (PedDisk** disk)
 {
@@ -1901,6 +1934,7 @@ static int
 do_fix_partition_order (PedDisk **disk)
 {
 	fix_partition_order(*disk);
+	return 1;
 }
 
 /* The next three are for lfdisk compatibility with fdisk */
@@ -2552,7 +2586,7 @@ build_option_table(void)
 static int
 _parse_options (int* argc_ptr, char*** argv_ptr)
 {
-  int opt;
+  int opt, align_type;
   struct option *options_table, *ptr;
 
   /* Build option table we need two pointers because getopt long
@@ -2595,6 +2629,17 @@ _parse_options (int* argc_ptr, char*** argv_ptr)
 	case 't':
 	  print_partition_types();
 	  break;
+#ifdef GNU_EXT
+	case 'A':
+	  align_type = string_to_align_type(optarg);
+	  if (align_type == ALIGNMENT_INVALID) {
+	      fdisk_usage_msg();
+	      return 0;
+	  }
+
+      current_partition_align_type = align_type;
+      break;
+#endif
 	case 'u':
 	  ped_unit_set_default(PED_UNIT_SECTOR);
 	  cylinder_unit = 0;
@@ -2626,11 +2671,14 @@ _parse_options (int* argc_ptr, char*** argv_ptr)
   *argv_ptr += optind;
   
   free(options_table);
-  
+ 
+	if(compat_mode && current_partition_align_type != ALIGNMENT_OPTIMAL)
+		{
+			fprintf(stderr, _("Warning: alignment option is ignored in "
+												"util-linux compatibile mode. Switch to cylinder alignment.\n"));
+		}	
+
   return 1;
-/* what is? */
-error:
-  return 0;
 }
 
 static PedDevice*
@@ -2771,8 +2819,9 @@ _init (int* argc_ptr, char*** argv_ptr)
  error_done_commands:
   _done_commands ();
   _done_messages ();
-  
- error_done_ui:
+ 
+ // unused	
+ //error_done_ui:
   fdisk_done_ui ();
   
  error:
@@ -2801,7 +2850,6 @@ int
 fdisk (int argc, char** argv)
 {
   PedDevice* dev;
-  PedDisk* disk;
   
   if (argc <= 1)
     {
